<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Teknisa\Model\Gestaopessoas\Entities\GpeQuadrovagas;
use HCM\Util\{ DateUtil, HCMParameters, Messages, Repositories, Util, NovoCodigo };

class QuadroVagasService {
    
    const CALCULATED_FIELDS = array('QTD_VINC_ATIVOS_OCUPACAO', 'QTD_VAGAS');
    const ON_SAVE_INVALIDATION_CHECKS = [
        ['action' => 'isValidadeFimInferiorInicio', 'invalidationMessage' => Messages::HCM00000_GPE_QUADROVAGAS_VALIDADE_INVALIDA],
        ['action' => 'isLimiteMenorIgualZero', 'invalidationMessage' => Messages::HCM00000_GPE_QUADROVAGAS_LIMITE_INVALIDO],
        ['action' => 'isEstruturaRepetida', 'invalidationMessage' => Messages::HCM00000_GPE_QUADROVAGAS_QUADRO_REPETIDO],
    ];

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function includeNewRowsData(array $rows): array {
        $alteredRows = $this->incluiCamposCalculadosRows($rows);
        return $alteredRows;
    }
    
    public function filterNewRowsData(array $rows, FilterCriteria $filterData): array {
        $filterConditions = Util::removerCaracterePorcentagemString($filterData->getConditions());
        foreach ($filterConditions as $filter) {
            $filterName = $filter['columnName'];
            $filterValue = $filter['value'];
            
            if (!in_array($filterName, self::CALCULATED_FIELDS)) {
                continue;
            }
            
            $rows = array_filter($rows, function ($item) use ($filterName, $filterValue) {
                $itemValue = $item[$filterName] ?? null;
                return $itemValue === intval($filterValue);
            });
        }
        
        return array_values($rows);
    }
    
    public function incluiCamposCalculadosRows(array $rows): array {
        $alteredRows = array_map('self::incluiCamposCalculadosRow', $rows);
        return $alteredRows;
    }
    
    public function incluiCamposCalculadosRow(array $row): array {
        $qntVinculosAtivosOcupEstrut = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getQntVinculosAtivosOcupEstrutHierarq(
            $row['NRORG'],
            $this->hcmParameters->getDtCompetencia(),
            $row['NROCUPACAOM'],
            $row['NRESTRUTURAM']
        );
        
        $row['QTD_VINC_ATIVOS_OCUPACAO'] = $qntVinculosAtivosOcupEstrut;
        $row['QTD_VAGAS'] = $row['QTDLIMITE'] - $row['QTD_VINC_ATIVOS_OCUPACAO'];
        $row['QTD_VAGAS'] = $row['QTD_VAGAS'] < 0 ? 0 : $row['QTD_VAGAS'];
        
        return $row;
    }
    
    public function areRowsInvalid(array $rows, DTO\Response $response): bool {
        foreach ($rows as $row) {
            if ($this->isRowInvalid($row, $response)) {
                return true;
            }
        }
        
        return false;
    }
    
    public function isRowInvalid(object $row, DTO\Response $response): bool {
        $isRowInvalid = false;
        
        foreach (self::ON_SAVE_INVALIDATION_CHECKS as $validation) {
            $isCheckInvalid = $validation['action'];
            if ($this->$isCheckInvalid($row)) {
                $isRowInvalid = true;
                $response->addMessage(new Message($validation['invalidationMessage'], DTO\Response\Message::TYPE_WARNING));
            }
        }
        
        return $isRowInvalid;
    }
    
    public function isValidadeFimInferiorInicio(object $row): bool {
        $dataIni = DateUtil::getDataDeString(substr($row['DTINIVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $dataFim = DateUtil::getDataDeString(substr($row['DTFIMVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        return !empty($dataFim) && $dataFim < $dataIni;
    }
    
    public function isLimiteMenorIgualZero(object $row): bool {
        $dataIni = $row['QTDLIMITE'];
        return $dataIni <= 0;
    }
    
    public function isEstruturaRepetida(object $row): bool {
        $quadrosEstruturasOcupacoesIguais = $this->getQuadrosPotencialmenteRepetidos($row);
        
        $iniValidadeRow = DateUtil::getDataDeString(substr($row['DTINIVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $fimValidadeRow = DateUtil::getDataDeString(substr($row['DTFIMVALIDADE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        
        foreach ($quadrosEstruturasOcupacoesIguais as $quadro) {
            $isPropostaIgual = !empty($quadro->getCdproposta()) && $quadro->getCdproposta() == $row['CDPROPOSTA'];
            $isPeriodosIntercedentes =
                (empty($fimValidadeRow) || $quadro->getDtinivalidade() <= $fimValidadeRow) && (empty($quadro->getDtfimvalidade()) || $quadro->getDtfimvalidade() >= $iniValidadeRow);
                
            if ($isPropostaIgual || $isPeriodosIntercedentes) {
                return true;
            }
        }
        
        return false;
    }
    
    public function getQuadrosPotencialmenteRepetidos(object $row): array {
        $quadrosEstruturasOcupacoesIguais = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->findBy(
            ['nrorg' => $row['NRORG'], 'nrestruturam' => $row['NRESTRUTURAM'], 'nrocupacaom' => $row['NROCUPACAOM'], 'idativo' => 'S']
        );
        
        $quadrosEstruturasOcupacoesIguais = array_filter(
            $quadrosEstruturasOcupacoesIguais,
            function ($item) use ($row) {
                return $item->getNrquadrovagas() != ($row['NRQUADROVAGAS'] ?? null);
            }
        );
        
        return $quadrosEstruturasOcupacoesIguais;
    }

    public function cadastroAutomatico($cdproposta, $dtinivalidade, $dtfimvalidade,$nrestruturam_list, $qtdlimite,$nrocupacaom, $nrorg, $nrquadrovagas, $addousubstituiQuadroVagas) {
        $quadrosDeVagasParaInserir = [];
        $msgRetorno = array();
        
        /* Validacoes iniciais */
        
        if ($qtdlimite <= 0) {
            $msgRetorno[] = array(
                'NRESTRUTURAM' => null,
                'msg' => "Nenhum quadro foi inserido. A quantidade limite deve ser superior à zero."
            );
            return $msgRetorno;
        }
        
        /* Criacao dos quadros */
        
        $this->entityManager->beginTransaction();
        
        try{
            foreach($nrestruturam_list as $estrutura){
                if($nrocupacaom){
                    foreach($nrocupacaom as $ocupacao) {
                        $quadroPotencialmenteIgual = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->findOneBy(array('nrorg' => $nrorg, 'nrestruturam' => $estrutura, 'nrocupacaom' => $ocupacao, 'idativo' => 'S', 'dtinivalidade' => $dtinivalidade));
                        
                        $estruturaQuadro = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg, 'nrestruturam' => $estrutura));
                        $nomeEstruturaQuadro = $estruturaQuadro != null ? $estruturaQuadro->getNmestruturam() : $estrutura;
                        $ocupacaoQuadro = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOM)->retornaOcupacao($nrorg, $ocupacao, $this->hcmParameters->getDtCompetencia());
                        $nomeOcupacaoQuadro = $ocupacaoQuadro != null && $ocupacaoQuadro->historico != null ? $ocupacaoQuadro->historico->getNmocupacaoh() : $ocupacao;
                        
                        /* Validacoes previas */
                        
                        if($quadroPotencialmenteIgual != null && $addousubstituiQuadroVagas == 'ADDCADAUTO'){
                            $msgRetorno[] = array(
                                'NRESTRUTURAM' => $estrutura,
                                'msg' => "Quadro de vagas não inserido para a razão social '$nomeEstruturaQuadro' porque ele já possui quadro de vagas com a ocupação '$nomeOcupacaoQuadro'!"
                            );
                            continue;
                        }
                        
                        $qntVinculosAtivosOcupEstrut = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getQntVinculosAtivosOcupEstrutHierarq(
                            $this->hcmParameters->getNrorgTrabalho(),
                            $this->hcmParameters->getDtCompetencia(),
                            $ocupacao,
                            $estrutura
                        );
                        
                        if ($qtdlimite < $qntVinculosAtivosOcupEstrut) {
                            $msgRetorno[] = array(
                                'NRESTRUTURAM' => $estrutura,
                                'msg' => "Quadro de vagas não inserido para a razão social '$nomeEstruturaQuadro' com a ocupação '$nomeOcupacaoQuadro'. A quantidade limite é inferior à quantidade de vínculos ativos na estrutura ($qntVinculosAtivosOcupEstrut)"
                            );
                            continue;
                        }
                        
                        /* Alteracao dos quadros */
                        
                        if($quadroPotencialmenteIgual != null && $addousubstituiQuadroVagas == 'SUBSTCADAUTO') {
                            $quadroPotencialmenteIgual->setDtinivalidade($dtinivalidade);
                            $quadroPotencialmenteIgual->setDtfimvalidade($dtfimvalidade);
                            $quadroPotencialmenteIgual->setQtdlimite($qtdlimite);
                            $quadroPotencialmenteIgual->setDtultatu(DateUtil::getDataAtual());
                            $quadroPotencialmenteIgual->setCdoperultatu($this->hcmParameters->getCdoperador());
                            $quadroPotencialmenteIgual->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                            $this->entityManager->persist($quadroPotencialmenteIgual);
                        }
                        else if($quadroPotencialmenteIgual == null) {
                            $quadrosDeVagasParaInserir[] = [
                                'nrorg' => $nrorg,
                                'estrutura' => $estrutura,
                                'ocupacao' => $ocupacao,
                                'dtinivalidade' => $dtinivalidade,
                                'dtfimvalidade'=> $dtfimvalidade,
                                'cdproposta' => $cdproposta,
                                'qtdlimite' => $qtdlimite
                            ];
                        } 
                    }
                }else{
                    $quadroPotencialmenteIgual = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->findOneBy(array('nrorg' => $nrorg, 'nrestruturam' => $estrutura, 'idativo' => 'S', 'dtinivalidade' => $dtinivalidade));
                    $estruturaQuadro = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg, 'nrestruturam' => $estrutura));
                    $nomeEstruturaQuadro = $estruturaQuadro != null ? $estruturaQuadro->getNmestruturam() : $estrutura;
                    /* Validacoes previas */
                    
                    if($quadroPotencialmenteIgual != null && $addousubstituiQuadroVagas == 'ADDCADAUTO'){
                        $msgRetorno[] = array(
                            'NRESTRUTURAM' => $estrutura,
                            'msg' => "Quadro de vagas não inserido para a razão social '$nomeEstruturaQuadro' porque ele já possui quadro de vagas"
                        );
                        continue;
                    }
                    
                    $qntVinculosAtivosOcupEstrut = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getQntVinculosAtivosOcupEstrutHierarq(
                        $this->hcmParameters->getNrorgTrabalho(),
                        $this->hcmParameters->getDtCompetencia(),
                        null,
                        $estrutura
                    );
                    
                    if ($qtdlimite < $qntVinculosAtivosOcupEstrut) {
                        $msgRetorno[] = array(
                            'NRESTRUTURAM' => $estrutura,
                            'msg' => "Quadro de vagas não inserido para a razão social '$nomeEstruturaQuadro' A quantidade limite é inferior à quantidade de vínculos ativos na estrutura ($qntVinculosAtivosOcupEstrut)"
                        );
                        continue;
                    }
                    
                    /* Alteracao dos quadros */
                    
                    if($quadroPotencialmenteIgual != null && $addousubstituiQuadroVagas == 'SUBSTCADAUTO') {
                        $quadroPotencialmenteIgual->setDtinivalidade($dtinivalidade);
                        $quadroPotencialmenteIgual->setDtfimvalidade($dtfimvalidade);
                        $quadroPotencialmenteIgual->setQtdlimite($qtdlimite);
                        $quadroPotencialmenteIgual->setDtultatu(DateUtil::getDataAtual());
                        $quadroPotencialmenteIgual->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $quadroPotencialmenteIgual->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->entityManager->persist($quadroPotencialmenteIgual);
                    }
                    else if($quadroPotencialmenteIgual == null) {
                        $quadrosDeVagasParaInserir[] = [
                            'nrorg' => $nrorg,
                            'estrutura' => $estrutura,
                            'dtinivalidade' => $dtinivalidade,
                            'dtfimvalidade'=> $dtfimvalidade,
                            'cdproposta' => $cdproposta,
                            'qtdlimite' => $qtdlimite
                        ];
                    } 
                } 
            }
            
            $this->geraNovoQuadroDeVagas($quadrosDeVagasParaInserir, $nrorg);
            $this->entityManager->flush();
            $this->entityManager->commit();           
            return $msgRetorno;
        }catch(\Exception $e){
            $this->entityManager->rollBack();
            throw $e;
        }
    }

    public function geraNovoQuadroDeVagas($quadrosDeVagasParaInserir, $nrorg) {
        foreach ($quadrosDeVagasParaInserir as $quadro) {
            $obj = new GpeQuadrovagas;
            $obj->setNrocupacaom($quadro['ocupacao']);
            $obj->setIdativo('S');
            $obj->setNrquadrovagas(NovoCodigo::geraCodigo($nrorg, 'GPE_QUADROVAGAS', 1, 12, true));
            $obj->setCdproposta($quadro['cdproposta']);
            $obj->setDtinivalidade($quadro['dtinivalidade']);
            $obj->setDtfimvalidade($quadro['dtfimvalidade']);
            $obj->setNrestruturam($quadro['estrutura']);
            $obj->setQtdlimite($quadro['qtdlimite']);
            $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($obj);
        }
    }
}